--[[
	Persistent Data for RaidTracker, Saved Variables	
]]


--***********************************************************************************
-- Saved Variables

KARaidTrackerDB = {
	Options = {
		AutoRaidCreation = 1,           -- on/off
		AutoBoss = 1,                   -- 0,1,2
		AutoBossBoss = "Trash mob",     -- just the name of the boss
		AutoZone = 1,                   -- on/off
		AskCost = 0,                    -- 1:poor, 2:common, 3:uncommon, 4:rare, 5:epic, 6:legendary, 7:artifact, asks for cost for items with at least this rarity
		MinQuality = 3,                 -- 1:poor, 2:common, 3:uncommon, 4:rare, 5:epic, 6:legendary, 7:artifact
		GroupItems = 0,                 -- 1:poor, 2:common, 3:uncommon, 4:rare, 5:epic, 6:legendary, 7:artifact
		GetDkpValue = 0,                -- 1:poor, 2:common, 3:uncommon, 4:rare, 5:epic, 6:legendary, 7:artifact
		Timezone = 0,
		TimeSync = 1,
		DebugFlag = nil,                -- on/off
		LogAttendees = 0,               -- 0:off, 1:for each looted item, 2:for each bosskill
		SaveExtendedPlayerInfo = 1,     -- on/off - save race, class and level  
		SaveTooltips = 1,               -- on/off - save tooltips of items
		TimeFormat = 1,                 -- on/off - Use 24h time format
		AutoBossChangeMinTime = 120,    -- how long should trash mob's ignored after a boss kill (seconds)?
		Wipe = 0,                       -- ask if the group dies if it is a wipe, if all are dead it will not ask
		WipePercent = 0.5,              -- how many prozent of group must be dead to ask
		WipeCoolDown = 120,             -- how long should death be ignored after a wipe count (seconds)
		NextBoss = 0,                   -- ask on boss kill whats the next boss is
		MaxLevel = 70,                  -- If player lvl is maxlevel it will not be exported to mldkp
		GuildSnapshot = 0,              -- Snapshots the guildroster on bosskill
		ExportFormat = 2,               -- ExportFormat for the xml string
		-- ML_RaidTracker_Custom_Options
		LogGroup = 1,
		GuildSnapshot = 0,
		AutoGroup = 1,
		LogBattlefield = 0,
		OldFormat = 1,
		MLdkp = 1,
		-- Internal flags
		VersionFix = 0,                 -- Flag to determine current database version for saved variables
		VersionCopy = 0,                -- Flag to determine current database version for saved variables
		TimeOffsetStatus = 0,           -- ???
		ItemOptionsSelected = nil,      -- ???
		CurrentRaid = nil,
	};	
    SortOptions = {
        method = "name",
        way = "asc",
        itemmethod = "looted",
        itemway = "asc",
        itemfilter = 1,
        playerraidway = "desc",
        playeritemfilter = 1,
        playeritemmethod = "name",
        playeritemway = "asc",
        itemhistorymethod = "name",
        itemhistoryway = "asc",
    };
    ItemOptions = { };
    CustomZoneTriggers = { };
    Online = { };
    RaidLog = { };
};

KA_RaidTracker_Online = KARaidTrackerDB.Online;
KA_RaidTracker_Options = KARaidTrackerDB.Options;
KA_RaidTracker_SortOptions = KARaidTrackerDB.SortOptions;
KA_RaidTracker_ItemOptions = KARaidTrackerDB.ItemOptions;
KA_RaidTracker_CustomZoneTriggers = KARaidTrackerDB.CustomZoneTriggers;
KA_RaidTracker_RaidLog = KARaidTrackerDB.RaidLog;


KARaidTrackerData = { db = KARaidTrackerDB }

function KARaidTrackerData:OnVariablesLoaded( )
	self.db = KARaidTrackerDB;
	
	self:RunVersionFix( );
	
	KA_RaidTracker_Online = self.db.Online;
	KA_RaidTracker_Options = self.db.Options;
	KA_RaidTracker_SortOptions = self.db.SortOptions;
	KA_RaidTracker_ItemOptions = self.db.ItemOptions;
	KA_RaidTracker_CustomZoneTriggers = self.db.CustomZoneTriggers;
	KA_RaidTracker_RaidLog = self.db.RaidLog;
end

function KARaidTrackerData:RunVersionFix( )
    local util = KARaidTrackerUtility

    local debugflagpre = self.db.Options.DebugFlag;
    --self.db.Options.DebugFlag = 1;  
    
    -- copy old db
    if(self.db.Options.VersionCopy == 0) then       
		DEFAULT_CHAT_FRAME:AddMessage("Starting CT_RaidTracker conversion", 1, 0.5, 0)
        if (CT_RaidTracker_RaidLog) then
        	self.db.RaidLog = util:CopyDeep(CT_RaidTracker_RaidLog);
        end;
        if (CT_RaidTracker_Options) then
        	self.db.Options = util:MergeDeep(self.db.Options, CT_RaidTracker_Options);       
        end;
        if (CT_RaidTracker_ItemOptions) then
        	self.db.ItemOptions = util:MergeDeep(self.db.ItemOptions, CT_RaidTracker_ItemOptions);       
        end;
        if (CT_RaidTracker_SortOptions) then
            self.db.SortOptions = util:MergeDeep(self.db.SortOptions, CT_RaidTracker_SortOptions);       
        end;
        if (CT_RaidTracker_CustomZoneTriggers) then
            self.db.CustomZoneTriggers = util:CopyDeep(CT_RaidTracker_CustomZoneTriggers);       
        end;
        if (CT_RaidTracker_Online) then
            self.db.Online = util:CopyDeep(CT_RaidTracker_Online);       
        end;
        if (CT_RaidTracker_VersionFix) then
        	self.db.Options.VersionFix = CT_RaidTracker_VersionFix;       
        end;
        if (CT_RaidTracker_GetCurrentRaid) then
        	self.db.Options.CurrentRaid = CT_RaidTracker_GetCurrentRaid;       
        end;
		DEFAULT_CHAT_FRAME:AddMessage("Done CT_RaidTracker conversion", 1, 0.5, 0)
        self.db.Options.VersionCopy = 1;
    end
	    
	-- Convert
	local opts = self.db.Options;
    local raidLog = self.db.RaidLog;
    
    if(opts.VersionFix == 0) then
        util:Debug("VersionFix", 1);
        for k, v in pairs(raidLog) do
            if(not raidLog[k]["PlayerInfos"]) then
                raidLog[k]["PlayerInfos"] = { };
            end
            if ( v["Notes"] ) then
                for notesk, notesv in pairs(v["Notes"]) do
                    if(not raidLog[k]["PlayerInfos"][notesk]) then
                        raidLog[k]["PlayerInfos"][notesk] = { };
                    end
                    raidLog[k]["PlayerInfos"][notesk]["note"] = notesv;
                    util:Debug("VersionFix", 1, "note", k, notesk, notesv);
                end
                raidLog[k]["Notes"] = nil;
            end
        end
        opts.VersionFix = 1;
    end
    if(opts.VersionFix == 1) then
        util:Debug("VersionFix", 2);
        opts.VersionFix = 2; -- Do not remove tooltips any longer
    end
    if(opts.VersionFix == 2) then
        util:Debug("VersionFix", 3);
        if(CT_RaidTracker_MinQuality) then opts.MinQuality = CT_RaidTracker_MinQuality; util:Debug("VersionFix", 3, "self.db.MinQuality", opts.MinQuality); end
        if(CT_RaidTracker_AutoRaidCreation) then opts.AutoRaidCreation = CT_RaidTracker_AutoRaidCreation; util:Debug("VersionFix", 3, "self.db.AutoRaidCreation", opts.AutoRaidCreation); end
        if(CT_RaidTracker_GroupItems) then opts.GroupItems = CT_RaidTracker_GroupItems; util:Debug("VersionFix", 3, "self.db.GroupItems", opts.GroupItems); end
        if(CT_RaidTracker_GetDkpValue) then opts.GetDkpValue = CT_RaidTracker_GetDkpValue; util:Debug("VersionFix", 3, "self.db.GetDkpValue", opts.GetDkpValue); end
        if(CT_RaidTracker_OldFormat) then opts.OldFormat = CT_RaidTracker_OldFormat; util:Debug("VersionFix", 3, "self.db.OldFormat", opts.OldFormat); end
        if(CT_RaidTracker_AutoBoss) then opts.AutoBoss = CT_RaidTracker_AutoBoss; util:Debug("VersionFix", 3, "self.db.AutoBoss", opts.AutoBoss); end
        if(CT_RaidTracker_AutoBossBoss) then opts.AutoBossBoss = CT_RaidTracker_AutoBossBoss; util:Debug("VersionFix", 3, "self.db.AutoBossBoss", opts.AutoBossBoss); end
        if(CT_RaidTracker_AutoZone) then opts.AutoZone = CT_RaidTracker_AutoZone; util:Debug("VersionFix", 3, "self.db.AutoZone", opts.AutoZone); end
        if(CT_RaidTracker_AskCosts) then opts.AskCost = CT_RaidTracker_AskCosts; util:Debug("VersionFix", 3, "self.db.AskCosts", opts.AskCosts); end
        if(CT_RaidTracker_Timezone) then opts.Timezone = CT_RaidTracker_Timezone; util:Debug("VersionFix", 3, "self.db.Timezone", opts..Timezone); end
        if(CT_RaidTracker_TimeSync) then opts.TimeSync = CT_RaidTracker_TimeSync; util:Debug("VersionFix", 3, "self.db.TimeSync", opts.TimeSync); end
        if(CT_RaidTracker_DebugFlag) then opts.DebugFlag = CT_RaidTracker_DebugFlag; util:Debug("VersionFix", 3, "self.db.DebugFlag", opts.DebugFlag); end
        
        for k, v in pairs(raidLog) do
            if(raidLog[k]["BossKills"] and getn(raidLog[k]["BossKills"]) >= 1 and not raidLog[k]["BossKills"][1]) then
                local tempbosskills = {};
                for k2, v2 in pairs(raidLog[k]["BossKills"]) do
                    tempbosskills[k2] = v2;
                end
                raidLog[k]["BossKills"] = {};
                for k2, v2 in pairs(tempbosskills) do
                    tinsert(raidLog[k]["BossKills"],
                        {
                            ["boss"] = k2,
                            ["time"] = v2,
                            ["attendees"] = {}
                        }
                    );
                    util:Debug("VersionFix", 3, "BossKills", k, k2);
                end
            end
        end
        opts.VersionFix = 3;
    end
    if(opts.VersionFix == 3) then
        util:Debug("VersionFix", 4);
        for k, v in pairs(raidLog) do
            if(raidLog[k]["BossKills"]) then
                for k2, v2 in pairs(raidLog[k]["BossKills"]) do
                    if(type(v2["time"]) == "table") then
                        util:Debug("VersionFix", 4, "BossKills Fix", k, k2, v2["time"]["boss"], v2["time"]["time"]);
                        raidLog[k]["BossKills"][k2]["boss"] = v2["time"]["boss"];
                        raidLog[k]["BossKills"][k2]["attendees"] = v2["time"]["attendees"];
                        raidLog[k]["BossKills"][k2]["time"] = v2["time"]["time"];
                    end
                end
            end
        end
        opts.VersionFix = 4;
    end
    if(opts.VersionFix == 4) then
        opts.Wipe = 1;                     -- ask if the group dies if it is a wipe, if all are dead it will not ask
        opts.WipePercent = 0.5;            -- how many prozent of group must be dead to ask
        opts.WipeCoolDown = 150;           -- how long should death be ignored after a wipe count (seconds)
        opts.NextBoss = 1;                 -- ask on boss kill whats the next boss is
        
        opts.VersionFix = 5;
    end
    if (opts.VersionFix == 5) then
        opts.MaxLevel = 70;                 -- If player lvl is maxlevel it will not be exported to mldkp
        opts.MLdkp = 1;                     -- Export für mldkp
        opts.GuildSnapshot = 0;             -- Snapshots the guildroster on bosskill
        
        opts.VersionFix = 6;
    end;
    if (opts.VersionFix == 6) then
        if (opts.MLdkp and opts.MLdkp == 1) then
            opts.ExportFormat = 2;
        else
            if (opts.OldFormat and opts.OldFormat == 1) then
                opts.ExportFormat = 0;
            else
                opts.ExportFormat = 1;
            end;
        end;
        opts.MLdkp = nil;
        opts.OldFormat = nil;
        opts.VersionFix = 7;
        KA_RaidTrackerOptionsFrame:Show();
    end;
    opts.DebugFlag = debugflagpre;
 end

